==> osam_flutter-3.2.6.tar.gz: lib/src/presentation/osam_provider.dart <==
import 'package:flutter/widgets.dart';
import 'package:osam/osam.dart';
import 'package:provider/provider.dart';
import 'package:provider/single_child_widget.dart';

part 'presenter.dart';

abstract class OsamProvider implements Widget {
  const factory OsamProvider({
    UseCaseContainer useCaseContainer,
    Object uiState,
    Object appState,
    int version,
    Key key,
    Widget child,
  }) = _OsamProvider;
}

class _OsamProvider extends InheritedWidget implements OsamProvider {
  const _OsamProvider(
      {this.useCaseContainer,
      this.appState,
      this.uiState,
      this.version,
      Key key,
      Widget child})
      : super(key: key, child: child);

  final UseCaseContainer useCaseContainer;
  final Object uiState;
  final Object appState;
  final int version;

  static _OsamProvider of<US, AS>(BuildContext context) =>
      context.dependOnInheritedWidgetOfExactType(aspect: _OsamProvider);

  @override
  bool updateShouldNotify(_OsamProvider oldWidget) => false;
}

class PersistLifecycleWrapper extends StatefulWidget {
  final Widget child;
  final Persist<Object, Object> persist;

  const PersistLifecycleWrapper(
      {Key key, @required this.child, @required this.persist})
      : super(key: key);

  @override
  _PersistLifecycleWrapperState createState() =>
      _PersistLifecycleWrapperState();
}

class _PersistLifecycleWrapperState extends State<PersistLifecycleWrapper>
    with WidgetsBindingObserver {
  @override
  void initState() {
    WidgetsBinding.instance.addObserver(this);
    super.initState();
  }

  @override
  Widget build(BuildContext context) => widget.child;

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.paused) {
      final osamProvider = _OsamProvider.of(context);
      widget.persist.save(
          state: osamProvider.appState,
          uiState: osamProvider.uiState,
          version: osamProvider.version);
    }
  }
}

mixin _PresenterProviderSingleChildWidget on SingleChildWidget {}

class PresenterProvider<P extends Presenter> extends SingleChildStatelessWidget
    with _PresenterProviderSingleChildWidget {
  final Widget child;
  final P presenter;

  const PresenterProvider({Key key, this.child, this.presenter})
      : super(key: key, child: child);

  static P of<P extends Presenter>(BuildContext context) =>
      Provider.of<P>(context, listen: false);

  @override
  Widget buildWithChild(BuildContext context, Widget child) {
    final useCaseProvider = _OsamProvider.of(context);
    return Provider<P>(
      create: (_) {
        presenter?._useCaseContainer = useCaseProvider.useCaseContainer;
        presenter?._uiState = useCaseProvider?.uiState;
        presenter?._appState = useCaseProvider?.appState;
        presenter?.init();
        return presenter;
      },
      child: child,
      dispose: (_, presenter) => presenter?.dispose(),
    );
  }
}

class MultiPresenterProvider extends StatelessWidget {
  final List<SingleChildWidget> providers;
  final Widget child;

  const MultiPresenterProvider(
      {Key key, @required this.providers, @required this.child})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: providers,
      child: child,
    );
  }
}

extension PresenterProviderExtension on BuildContext {
  P presenter<P extends Presenter>() => PresenterProvider.of<P>(this);
}

==> osam_flutter-3.4.4.tar.gz: lib/src/presentation/osam_provider.dart <==
import 'package:flutter/widgets.dart';
import 'package:osam/osam.dart';
import 'package:provider/provider.dart';
import 'package:provider/single_child_widget.dart';

part 'presenter.dart';

abstract class OsamProvider implements Widget {
  const factory OsamProvider({
    UseCaseContainer useCaseContainer,
    Object uiState,
    Object appState,
    int version,
    Key key,
    Widget child,
  }) = _OsamProvider;
}

class _OsamProvider extends InheritedWidget implements OsamProvider {
  const _OsamProvider(
      {this.useCaseContainer,
      this.appState,
      this.uiState,
      this.version,
      Key key,
      Widget child})
      : super(key: key, child: child);

  final UseCaseContainer useCaseContainer;
  final Object uiState;
  final Object appState;
  final int version;

  static _OsamProvider of<US, AS>(BuildContext context) =>
      context.dependOnInheritedWidgetOfExactType(aspect: _OsamProvider);

  @override
  bool updateShouldNotify(_OsamProvider oldWidget) => false;
}

class PersistLifecycleWrapper extends StatefulWidget {
  final Widget child;
  final Persist<Object, Object> persist;

  const PersistLifecycleWrapper(
      {Key key, @required this.child, @required this.persist})
      : super(key: key);

  @override
  _PersistLifecycleWrapperState createState() =>
      _PersistLifecycleWrapperState();
}

class _PersistLifecycleWrapperState extends State<PersistLifecycleWrapper>
    with WidgetsBindingObserver {
  @override
  void initState() {
    WidgetsBinding.instance.addObserver(this);
    super.initState();
  }

  @override
  Widget build(BuildContext context) => widget.child;

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.paused ||
        state == AppLifecycleState.inactive) {
      final osamProvider = _OsamProvider.of(context);
      widget.persist.save(
          state: osamProvider.appState,
          uiState: osamProvider.uiState,
          version: osamProvider.version);
    }
  }
}

mixin _PresenterProviderSingleChildWidget on SingleChildWidget {}

class PresenterProvider<P extends Presenter> extends SingleChildStatelessWidget
    with _PresenterProviderSingleChildWidget {
  final Widget child;
  final P presenter;

  const PresenterProvider({Key key, this.child, this.presenter})
      : super(key: key, child: child);

  static P of<P extends Presenter>(BuildContext context) =>
      Provider.of<P>(context, listen: false);

  @override
  Widget buildWithChild(BuildContext context, Widget child) {
    final useCaseProvider = _OsamProvider.of(context);
    return Provider<P>(
      create: (_) {
        presenter?._useCaseContainer = useCaseProvider.useCaseContainer;
        presenter?._uiState = useCaseProvider?.uiState;
        presenter?._appState = useCaseProvider?.appState;
        presenter?.init();
        return presenter;
      },
      child: child,
      dispose: (_, presenter) => presenter?.dispose(),
    );
  }
}

class MultiPresenterProvider extends StatelessWidget {
  final List<SingleChildWidget> providers;
  final Widget child;

  const MultiPresenterProvider(
      {Key key, @required this.providers, @required this.child})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: providers,
      child: child,
    );
  }
}

extension PresenterProviderExtension on BuildContext {
  P presenter<P extends Presenter>() => PresenterProvider.of<P>(this);
}
